@@ -3,42 +3,25 @@ Tests for L{pyflakes.scripts.pyflakes}.
 """
 
 import contextlib
+import io
 import os
 import sys
 import shutil
 import subprocess
 import tempfile
 
+from pyflakes.checker import PYPY
 from pyflakes.messages import UnusedImport
 from pyflakes.reporter import Reporter
 from pyflakes.api import (
     main,
+    check,
     checkPath,
     checkRecursive,
     iterSourceCode,
 )
 from pyflakes.test.harness import TestCase, skipIf
 
-if sys.version_info < (3,):
-    from cStringIO import StringIO
-else:
-    from io import StringIO
-    unichr = chr
-
-try:
-    sys.pypy_version_info
-    PYPY = True
-except AttributeError:
-    PYPY = False
-
-try:
-    WindowsError
-    WIN = True
-except NameError:
-    WIN = False
-
-ERROR_HAS_COL_NUM = ERROR_HAS_LAST_LINE = sys.version_info >= (3, 2) or PYPY
-
 
 def withStderrTo(stderr, f, *args, **kwargs):
     """
@@ -51,7 +34,7 @@ def withStderrTo(stderr, f, *args, **kwargs):
         sys.stderr = outer
 
 
-class Node(object):
+class Node:
     """
     Mock an AST node.
     """
@@ -60,32 +43,14 @@ class Node(object):
         self.col_offset = col_offset
 
 
-class SysStreamCapturing(object):
-
-    """
-    Context manager capturing sys.stdin, sys.stdout and sys.stderr.
+class SysStreamCapturing:
+    """Context manager capturing sys.stdin, sys.stdout and sys.stderr.
 
     The file handles are replaced with a StringIO object.
-    On environments that support it, the StringIO object uses newlines
-    set to os.linesep.  Otherwise newlines are converted from \\n to
-    os.linesep during __exit__.
     """
 
-    def _create_StringIO(self, buffer=None):
-        # Python 3 has a newline argument
-        try:
-            return StringIO(buffer, newline=os.linesep)
-        except TypeError:
-            self._newline = True
-            # Python 2 creates an input only stream when buffer is not None
-            if buffer is None:
-                return StringIO()
-            else:
-                return StringIO(buffer)
-
     def __init__(self, stdin):
-        self._newline = False
-        self._stdin = self._create_StringIO(stdin or '')
+        self._stdin = io.StringIO(stdin or '', newline=os.linesep)
 
     def __enter__(self):
         self._orig_stdin = sys.stdin
@@ -93,8 +58,8 @@ class SysStreamCapturing(object):
         self._orig_stderr = sys.stderr
 
         sys.stdin = self._stdin
-        sys.stdout = self._stdout_stringio = self._create_StringIO()
-        sys.stderr = self._stderr_stringio = self._create_StringIO()
+        sys.stdout = self._stdout_stringio = io.StringIO(newline=os.linesep)
+        sys.stderr = self._stderr_stringio = io.StringIO(newline=os.linesep)
 
         return self
 
@@ -102,16 +67,12 @@ class SysStreamCapturing(object):
         self.output = self._stdout_stringio.getvalue()
         self.error = self._stderr_stringio.getvalue()
 
-        if self._newline and os.linesep != '\n':
-            self.output = self.output.replace('\n', os.linesep)
-            self.error = self.error.replace('\n', os.linesep)
-
         sys.stdin = self._orig_stdin
         sys.stdout = self._orig_stdout
         sys.stderr = self._orig_stderr
 
 
-class LoggingReporter(object):
+class LoggingReporter:
     """
     Implementation of Reporter that just appends any error to a list.
     """
@@ -201,10 +162,6 @@ class TestIterSourceCode(TestCase):
         with open(os.path.join(self.tempdir, 'c'), 'w') as fd:
             fd.write('hello\nworld\n')
 
-        python2 = os.path.join(self.tempdir, 'd')
-        with open(python2, 'w') as fd:
-            fd.write('#!/usr/bin/env python2\n')
-
         python3 = os.path.join(self.tempdir, 'e')
         with open(python3, 'w') as fd:
             fd.write('#!/usr/bin/env python3\n')
@@ -217,10 +174,6 @@ class TestIterSourceCode(TestCase):
         with open(python3args, 'w') as fd:
             fd.write('#!/usr/bin/python3 -u\n')
 
-        python2u = os.path.join(self.tempdir, 'h')
-        with open(python2u, 'w') as fd:
-            fd.write('#!/usr/bin/python2u\n')
-
         python3d = os.path.join(self.tempdir, 'i')
         with open(python3d, 'w') as fd:
             fd.write('#!/usr/local/bin/python3d\n')
@@ -229,10 +182,6 @@ class TestIterSourceCode(TestCase):
         with open(python38m, 'w') as fd:
             fd.write('#! /usr/bin/env python3.8m\n')
 
-        python27 = os.path.join(self.tempdir, 'k')
-        with open(python27, 'w') as fd:
-            fd.write('#!/usr/bin/python2.7   \n')
-
         # Should NOT be treated as Python source
         notfirst = os.path.join(self.tempdir, 'l')
         with open(notfirst, 'w') as fd:
@@ -240,8 +189,10 @@ class TestIterSourceCode(TestCase):
 
         self.assertEqual(
             sorted(iterSourceCode([self.tempdir])),
-            sorted([python, python2, python3, pythonw, python3args, python2u,
-                    python3d, python38m, python27]))
+            sorted([
+                python, python3, pythonw, python3args, python3d,
+                python38m,
+            ]))
 
     def test_multipleDirectories(self):
         """
@@ -280,11 +231,9 @@ class TestReporter(TestCase):
         number, error message, actual line of source and a caret pointing to
         where the error is.
         """
-        err = StringIO()
+        err = io.StringIO()
         reporter = Reporter(None, err)
-        reporter.syntaxError('foo.py', 'a problem', 3,
-                             8 if sys.version_info >= (3, 8) else 7,
-                             'bad line of source')
+        reporter.syntaxError('foo.py', 'a problem', 3, 8, 'bad line of source')
         self.assertEqual(
             ("foo.py:3:8: a problem\n"
              "bad line of source\n"
@@ -296,7 +245,7 @@ class TestReporter(TestCase):
         C{syntaxError} doesn't include a caret pointing to the error if
         C{offset} is passed as C{None}.
         """
-        err = StringIO()
+        err = io.StringIO()
         reporter = Reporter(None, err)
         reporter.syntaxError('foo.py', 'a problem', 3, None,
                              'bad line of source')
@@ -305,13 +254,24 @@ class TestReporter(TestCase):
              "bad line of source\n"),
             err.getvalue())
 
+    def test_syntaxErrorNoText(self):
+        """
+        C{syntaxError} doesn't include text or nonsensical offsets if C{text} is C{None}.
+
+        This typically happens when reporting syntax errors from stdin.
+        """
+        err = io.StringIO()
+        reporter = Reporter(None, err)
+        reporter.syntaxError('<stdin>', 'a problem', 0, 0, None)
+        self.assertEqual(("<stdin>:1:1: a problem\n"), err.getvalue())
+
     def test_multiLineSyntaxError(self):
         """
         If there's a multi-line syntax error, then we only report the last
         line.  The offset is adjusted so that it is relative to the start of
         the last line.
         """
-        err = StringIO()
+        err = io.StringIO()
         lines = [
             'bad line of source',
             'more bad lines of source',
@@ -319,18 +279,17 @@ class TestReporter(TestCase):
         reporter = Reporter(None, err)
         reporter.syntaxError('foo.py', 'a problem', 3, len(lines[0]) + 7,
                              '\n'.join(lines))
-        column = 25 if sys.version_info >= (3, 8) else 7
         self.assertEqual(
-            ("foo.py:3:%d: a problem\n" % column +
+            ("foo.py:3:25: a problem\n" +
              lines[-1] + "\n" +
-             " " * (column - 1) + "^\n"),
+             " " * 24 + "^\n"),
             err.getvalue())
 
     def test_unexpectedError(self):
         """
         C{unexpectedError} reports an error processing a source file.
         """
-        err = StringIO()
+        err = io.StringIO()
         reporter = Reporter(None, err)
         reporter.unexpectedError('source.py', 'error message')
         self.assertEqual('source.py: error message\n', err.getvalue())
@@ -340,11 +299,11 @@ class TestReporter(TestCase):
         C{flake} reports a code warning from Pyflakes.  It is exactly the
         str() of a L{pyflakes.messages.Message}.
         """
-        out = StringIO()
+        out = io.StringIO()
         reporter = Reporter(out, None)
         message = UnusedImport('foo.py', Node(42), 'bar')
         reporter.flake(message)
-        self.assertEqual(out.getvalue(), "%s\n" % (message,))
+        self.assertEqual(out.getvalue(), f"{message}\n")
 
 
 class CheckTests(TestCase):
@@ -374,7 +333,7 @@ class CheckTests(TestCase):
         @param path: A path to a file to check.
         @param errorList: A list of errors expected to be printed to stderr.
         """
-        err = StringIO()
+        err = io.StringIO()
         count = withStderrTo(err, checkPath, path)
         self.assertEqual(
             (count, err.getvalue()), (len(errorList), ''.join(errorList)))
@@ -439,9 +398,8 @@ def baz():
             exec(source)
         try:
             evaluate(source)
-        except SyntaxError:
-            e = sys.exc_info()[1]
-            if not PYPY:
+        except SyntaxError as e:
+            if not PYPY and sys.version_info < (3, 10):
                 self.assertTrue(e.text.count('\n') > 1)
         else:
             self.fail()
@@ -449,10 +407,15 @@ def baz():
         with self.makeTempFile(source) as sourcePath:
             if PYPY:
                 message = 'end of file (EOF) while scanning triple-quoted string literal'
+            elif sys.version_info >= (3, 10):
+                message = 'unterminated triple-quoted string literal (detected at line 8)'  # noqa: E501
             else:
                 message = 'invalid syntax'
 
-            column = 8 if sys.version_info >= (3, 8) else 11
+            if PYPY or sys.version_info >= (3, 10):
+                column = 12
+            else:
+                column = 8
             self.assertHasErrors(
                 sourcePath,
                 ["""\
@@ -468,21 +431,23 @@ def baz():
         """
         with self.makeTempFile("def foo(") as sourcePath:
             if PYPY:
-                result = """\
-%s:1:7: parenthesis is never closed
-def foo(
-      ^
-""" % (sourcePath,)
+                msg = 'parenthesis is never closed'
+            elif sys.version_info >= (3, 10):
+                msg = "'(' was never closed"
             else:
-                result = """\
-%s:1:9: unexpected EOF while parsing
-def foo(
-        ^
-""" % (sourcePath,)
+                msg = 'unexpected EOF while parsing'
 
-            self.assertHasErrors(
-                sourcePath,
-                [result])
+            if PYPY or sys.version_info >= (3, 10):
+                column = 8
+            else:
+                column = 9
+
+            spaces = ' ' * (column - 1)
+            expected = '{}:1:{}: {}\ndef foo(\n{}^\n'.format(
+                sourcePath, column, msg, spaces
+            )
+
+            self.assertHasErrors(sourcePath, [expected])
 
     def test_eofSyntaxErrorWithTab(self):
         """
@@ -490,16 +455,13 @@ def foo(
         syntax error reflects the cause for the syntax error.
         """
         with self.makeTempFile("if True:\n\tfoo =") as sourcePath:
-            column = 6 if PYPY else 7
-            last_line = '\t    ^' if PYPY else '\t     ^'
-
             self.assertHasErrors(
                 sourcePath,
-                ["""\
-%s:2:%s: invalid syntax
+                [f"""\
+{sourcePath}:2:7: invalid syntax
 \tfoo =
-%s
-""" % (sourcePath, column, last_line)])
+\t     ^
+"""])
 
     def test_nonDefaultFollowsDefaultSyntaxError(self):
         """
@@ -512,25 +474,29 @@ def foo(bar=baz, bax):
     pass
 """
         with self.makeTempFile(source) as sourcePath:
-            if ERROR_HAS_LAST_LINE:
-                if PYPY:
-                    column = 7
-                elif sys.version_info >= (3, 9):
-                    column = 21
-                elif sys.version_info >= (3, 8):
-                    column = 9
-                else:
-                    column = 8
-                last_line = ' ' * (column - 1) + '^\n'
-                columnstr = '%d:' % column
+            if sys.version_info >= (3, 12):
+                msg = 'parameter without a default follows parameter with a default'  # noqa: E501
             else:
-                last_line = columnstr = ''
+                msg = 'non-default argument follows default argument'
+
+            if PYPY and sys.version_info >= (3, 9):
+                column = 18
+            elif PYPY:
+                column = 8
+            elif sys.version_info >= (3, 10):
+                column = 18
+            elif sys.version_info >= (3, 9):
+                column = 21
+            else:
+                column = 9
+            last_line = ' ' * (column - 1) + '^\n'
             self.assertHasErrors(
                 sourcePath,
-                ["""\
-%s:1:%s non-default argument follows default argument
+                [f"""\
+{sourcePath}:1:{column}: {msg}
 def foo(bar=baz, bax):
-%s""" % (sourcePath, columnstr, last_line)])
+{last_line}"""]
+            )
 
     def test_nonKeywordAfterKeywordSyntaxError(self):
         """
@@ -542,59 +508,43 @@ def foo(bar=baz, bax):
 foo(bar=baz, bax)
 """
         with self.makeTempFile(source) as sourcePath:
-            if ERROR_HAS_LAST_LINE:
-                if PYPY:
-                    column = 12
-                elif sys.version_info >= (3, 9):
-                    column = 17
-                elif sys.version_info >= (3, 8):
-                    column = 14
-                else:
-                    column = 13
-                last_line = ' ' * (column - 1) + '^\n'
-                columnstr = '%d:' % column
+            if sys.version_info >= (3, 9):
+                column = 17
+            elif not PYPY:
+                column = 14
             else:
-                last_line = columnstr = ''
+                column = 13
+            last_line = ' ' * (column - 1) + '^\n'
+            columnstr = '%d:' % column
 
-            if sys.version_info >= (3, 5):
-                message = 'positional argument follows keyword argument'
-            else:
-                message = 'non-keyword arg after keyword arg'
+            message = 'positional argument follows keyword argument'
 
             self.assertHasErrors(
                 sourcePath,
                 ["""\
-%s:1:%s %s
+{}:1:{} {}
 foo(bar=baz, bax)
-%s""" % (sourcePath, columnstr, message, last_line)])
+{}""".format(sourcePath, columnstr, message, last_line)])
 
     def test_invalidEscape(self):
         """
         The invalid escape syntax raises ValueError in Python 2
         """
-        ver = sys.version_info
         # ValueError: invalid \x escape
         with self.makeTempFile(r"foo = '\xyz'") as sourcePath:
-            if ver < (3,):
-                decoding_error = "%s: problem decoding source\n" % (sourcePath,)
+            position_end = 1
+            if PYPY and sys.version_info >= (3, 9):
+                column = 7
+            elif PYPY:
+                column = 6
+            elif (3, 9) <= sys.version_info < (3, 12):
+                column = 13
             else:
-                position_end = 1
-                if PYPY:
-                    column = 5
-                elif ver >= (3, 9):
-                    column = 13
-                else:
-                    column = 7
-                    # Column has been "fixed" since 3.2.4 and 3.3.1
-                    if ver < (3, 2, 4) or ver[:3] == (3, 3, 0):
-                        position_end = 2
+                column = 7
 
-                if ERROR_HAS_LAST_LINE:
-                    last_line = '%s^\n' % (' ' * (column - 1))
-                else:
-                    last_line = ''
+            last_line = '%s^\n' % (' ' * (column - 1))
 
-                decoding_error = """\
+            decoding_error = """\
 %s:1:%d: (unicode error) 'unicodeescape' codec can't decode bytes \
 in position 0-%d: truncated \\xXX escape
 foo = '\\xyz'
@@ -635,7 +585,7 @@ foo = '\\xyz'
         """
         If source file declares the correct encoding, no error is reported.
         """
-        SNOWMAN = unichr(0x2603)
+        SNOWMAN = chr(0x2603)
         source = ("""\
 # coding: utf-8
 x = "%s"
@@ -655,40 +605,33 @@ x = "%s"
         If a source file contains bytes which cannot be decoded, this is
         reported on stderr.
         """
-        SNOWMAN = unichr(0x2603)
+        SNOWMAN = chr(0x2603)
         source = ("""\
 # coding: ascii
 x = "%s"
 """ % SNOWMAN).encode('utf-8')
         with self.makeTempFile(source) as sourcePath:
-            if PYPY and sys.version_info < (3, ):
-                message = ('\'ascii\' codec can\'t decode byte 0xe2 '
-                           'in position 21: ordinal not in range(128)')
-                result = """\
-%s:0:0: %s
-x = "\xe2\x98\x83"
-        ^\n""" % (sourcePath, message)
-
-            else:
-                message = 'problem decoding source'
-                result = "%s: problem decoding source\n" % (sourcePath,)
-
             self.assertHasErrors(
-                sourcePath, [result])
+                sourcePath,
+                [f"{sourcePath}:1:1: 'ascii' codec can't decode byte 0xe2 in position 21: ordinal not in range(128)\n"])  # noqa: E501
 
     def test_misencodedFileUTF16(self):
         """
         If a source file contains bytes which cannot be decoded, this is
         reported on stderr.
         """
-        SNOWMAN = unichr(0x2603)
+        SNOWMAN = chr(0x2603)
         source = ("""\
 # coding: ascii
 x = "%s"
 """ % SNOWMAN).encode('utf-16')
         with self.makeTempFile(source) as sourcePath:
-            self.assertHasErrors(
-                sourcePath, ["%s: problem decoding source\n" % (sourcePath,)])
+            if sys.version_info < (3, 11, 4):
+                expected = f"{sourcePath}: problem decoding source\n"
+            else:
+                expected = f"{sourcePath}:1: source code string cannot contain null bytes\n"  # noqa: E501
+
+            self.assertHasErrors(sourcePath, [expected])
 
     def test_checkRecursive(self):
         """
@@ -700,10 +643,10 @@ x = "%s"
             os.mkdir(os.path.join(tempdir, 'foo'))
             file1 = os.path.join(tempdir, 'foo', 'bar.py')
             with open(file1, 'wb') as fd:
-                fd.write("import baz\n".encode('ascii'))
+                fd.write(b"import baz\n")
             file2 = os.path.join(tempdir, 'baz.py')
             with open(file2, 'wb') as fd:
-                fd.write("import contraband".encode('ascii'))
+                fd.write(b"import contraband")
             log = []
             reporter = LoggingReporter(log)
             warnings = checkRecursive([tempdir], reporter)
@@ -716,6 +659,35 @@ x = "%s"
         finally:
             shutil.rmtree(tempdir)
 
+    def test_stdinReportsErrors(self):
+        """
+        L{check} reports syntax errors from stdin
+        """
+        source = "max(1 for i in range(10), key=lambda x: x+1)\n"
+        err = io.StringIO()
+        count = withStderrTo(err, check, source, "<stdin>")
+        self.assertEqual(count, 1)
+        errlines = err.getvalue().split("\n")[:-1]
+
+        if sys.version_info >= (3, 9):
+            expected_error = [
+                "<stdin>:1:5: Generator expression must be parenthesized",
+                "max(1 for i in range(10), key=lambda x: x+1)",
+                "    ^",
+            ]
+        elif PYPY:
+            expected_error = [
+                "<stdin>:1:4: Generator expression must be parenthesized if not sole argument",  # noqa: E501
+                "max(1 for i in range(10), key=lambda x: x+1)",
+                "   ^",
+            ]
+        else:
+            expected_error = [
+                "<stdin>:1:5: Generator expression must be parenthesized",
+            ]
+
+        self.assertEqual(errlines, expected_error)
+
 
 class IntegrationTests(TestCase):
     """
@@ -758,9 +730,8 @@ class IntegrationTests(TestCase):
                                  stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             (stdout, stderr) = p.communicate()
         rv = p.wait()
-        if sys.version_info >= (3,):
-            stdout = stdout.decode('utf-8')
-            stderr = stderr.decode('utf-8')
+        stdout = stdout.decode('utf-8')
+        stderr = stderr.decode('utf-8')
         return (stdout, stderr, rv)
 
     def test_goodFile(self):
@@ -778,10 +749,10 @@ class IntegrationTests(TestCase):
         and the warnings are printed to stdout.
         """
         with open(self.tempfilepath, 'wb') as fd:
-            fd.write("import contraband\n".encode('ascii'))
+            fd.write(b"import contraband\n")
         d = self.runPyflakes([self.tempfilepath])
         expected = UnusedImport(self.tempfilepath, Node(1), 'contraband')
-        self.assertEqual(d, ("%s%s" % (expected, os.linesep), '', 1))
+        self.assertEqual(d, (f"{expected}{os.linesep}", '', 1))
 
     def test_errors_io(self):
         """
@@ -790,8 +761,8 @@ class IntegrationTests(TestCase):
         printed to stderr.
         """
         d = self.runPyflakes([self.tempfilepath])
-        error_msg = '%s: No such file or directory%s' % (self.tempfilepath,
-                                                         os.linesep)
+        error_msg = '{}: No such file or directory{}'.format(self.tempfilepath,
+                                                             os.linesep)
         self.assertEqual(d, ('', error_msg, 1))
 
     def test_errors_syntax(self):
@@ -801,10 +772,10 @@ class IntegrationTests(TestCase):
         printed to stderr.
         """
         with open(self.tempfilepath, 'wb') as fd:
-            fd.write("import".encode('ascii'))
+            fd.write(b"import")
         d = self.runPyflakes([self.tempfilepath])
-        error_msg = '{0}:1:{2}: invalid syntax{1}import{1}     {3}^{1}'.format(
-            self.tempfilepath, os.linesep, 6 if PYPY else 7, '' if PYPY else ' ')
+        error_msg = '{0}:1:7: invalid syntax{1}import{1}      ^{1}'.format(
+            self.tempfilepath, os.linesep)
         self.assertEqual(d, ('', error_msg, 1))
 
     def test_readFromStdin(self):
@@ -813,7 +784,7 @@ class IntegrationTests(TestCase):
         """
         d = self.runPyflakes([], stdin='import contraband')
         expected = UnusedImport('<stdin>', Node(1), 'contraband')
-        self.assertEqual(d, ("%s%s" % (expected, os.linesep), '', 1))
+        self.assertEqual(d, (f"{expected}{os.linesep}", '', 1))
 
 
 class TestMain(IntegrationTests):
